/* ==========================================================================
   #SCRIPTS
   ========================================================================== */

// Dev settings - set `isDev` to false by default.
var isDev = false;

// Helper function to check if an element exists in the DOM.
$.fn.exists = function(){ return this.length>0; };

// Helper function to determine when/if an element enters or leaves the viewport
// to then trigger an action/event.
function isScrolledIntoView(elem) {
	var $elem = $(elem);
	var $window = $(window);

	var docViewTop = $window.scrollTop();
	var docViewBottom = docViewTop + $window.height();

	var elemTop = $elem.offset().top;
	var elemBottom = elemTop + $elem.height();

	return ((elemBottom <= docViewBottom) && (elemTop >= docViewTop));
}

function getParameterByName(name, url) {
    if (!url) url = window.location.href;
    name = name.replace(/[\[\]]/g, "\\$&");
    var regex = new RegExp("[?&]" + name + "(=([^&#]*)|&|#|$)"),
    results = regex.exec(url);
    if (!results) return null;
    if (!results[2]) return '';
    return decodeURIComponent(results[2].replace(/\+/g, " "));
}

// Function from David Walsh: http://davidwalsh.name/css-animation-callback
// make sure browser is listening for only one version of event.
function whichAnimationEvent() {
	var t,
el = document.createElement("fakeelement");
	var animations = {
			"animation": "animationend",
			"OAnimation": "oAnimationEnd",
			"MozAnimation": "animationend",
			"WebkitAnimation": "webkitAnimationEnd"
	}
	for (t in animations) {
			if (el.style[t] !== undefined) {
					return animations[t];
			}
	}
}
var animationEvent = whichAnimationEvent();

function whichTransitionEvent() {
	var t,
			el = document.createElement("fakeelement");
	var transitions = {
			"transition": "transitionend",
			"OTransition": "oTransitionEnd",
			"MozTransition": "transitionend",
			"WebkitTransition": "webkitTransitionEnd"
	}

	for (t in transitions) {
			if (el.style[t] !== undefined) {
					return transitions[t];
			}
	}
}
var transitionEvent = whichTransitionEvent();


// A function that can be called from the app to empty the elements in a timeline
// or library container to avoid multiple insertions fo the JSON content.
function clearContent(pDiv) {
	pDiv = $(pDiv);
	if(!pDiv.exists()) {
		pDiv = $("#timeline");
	}
	pDiv.empty();
}


// Initialize the jQuery UI tabs on the coaching and library timeline.
// Set the active tab if it's a push notification, defaults to coaching tab.
function initTabs(tab) {

	$("#toggleView").tabs({
		show: { effect: "fadeIn", duration: 200 },
		hide: { effect: "fadeOut", duration: 200 },
		active: tab
	});

}


// Set the height of card elements on the same row in the coaching and library
// timelines. Only applies at 768px wide and above.

function setTileHeight() {

	if ($('.eqHeight').exists()) {
		$('.eqHeight .c-card__heading').matchHeight();
		$('.c-card').matchHeight();
	}

}


/* Article mappings
	 ========================================================================== */

// Link the article cards that appear in the library to the correct article page.

// Coaching article url mappings
var articleAssets = {
	"results":[
		{
			"ArticleAssetUrl":[
				{
					"Id":117,
					"MediaAsset":{
						"MediaAssetId":56,
						"Title":"Humidification",
						"Url":"articles/html_coaching_article_humidification.html"
					}
				},
				{
					"Id":118,
					"MediaAsset":{
						"MediaAssetId":57,
						"Title":"Ramp time",
						"Url":"articles/html_coaching_article_ramp_time.html"
					}
				},
				{
					"Id":123,
					"MediaAsset":{
						"MediaAssetId":58,
						"Title":"Redness and skin discomfort",
						"Url":"articles/html_coaching_article_redness_skin_discomfort.html"
					}
				},
				{
					"Id":10133,
					"MediaAsset":{
						"MediaAssetId":10056,
						"Title":"Nasal congestion",
						"Url":"articles/html_coaching_article_nasal_congestion.html"
					}
				},
				{
					"Id":10134,
					"MediaAsset":{
						"MediaAssetId":10057,
						"Title":"Getting used to treatment",
						"Url":"articles/html_coaching_article_getting_used_to_treatment.html"
					}
				}
			]
		}
	]
}

// Use the `MediaAssetId` parameter to find the matching article to link to
// in the artclee assets array above.
function getMatchingArticleUrl(MediaAssetId) {
	var articles = articleAssets.results[0].ArticleAssetUrl;
	for(var i = 0; i < articles.length; i++) {
		var obj = articles[i];
		if(obj.MediaAsset.MediaAssetId == MediaAssetId) {
			return obj.MediaAsset.Url;
		}
	}
}



/* Anchor functions
	 ========================================================================== */

// Push notification scroll to asset in timeline
function scrollToAsset(assetId, delay) {
	delay = delay || 350;
	assetId = $("[id*="+ assetId +"]");

	if (assetId.exists()) {
		var buffer = 0;
		if ($(".c-nav--tabs").exists()) {
			buffer = $(".c-nav--tabs").outerHeight();
        }

		setTimeout(function() {
			$('html,body').animate({ scrollTop: assetId.offset().top - buffer }, 350, 'swing').promise().then(function() {

				// If this is an FAQ item, trigger the expander element to open
				if ((assetId.attr("id")).substring(0, 3) == "FAQ") {
					assetId.find(".c-expander__label").trigger("click");
				}

			});
		}, delay);

	}
}


// Smooth scroll back to the top of the web page.
function backToTop(target, buffer) {
	// Set defaults if no target or buffer parameters are passed.
	target = (typeof(target)==='undefined') ? "body" : "#" + target;
	if (typeof(buffer)==='undefined') buffer = 0;

	$('html, body').animate({ scrollTop: $(target).offset().top - buffer }, 350, 'swing');
}



// Smooth scroll to an element on the page.
function scrollToElem(elem) {
	// Grab the target element we want to scroll to from the href attribute.
	var target = $(elem).attr('href');

	setTimeout(function() {
			backToTop(target, 60);
	}, 350);

}


/* Expander component functions
   ========================================================================== */
// Toggle an expander component on the page. Use it for testing or
// development if expanders will be controlled from the back-end.

function res_expander() {
	$('.c-expander').children('.c-expander__label').on('click', function (e) {
			e.preventDefault();
			var label_elem          = $(this),
					elem_parent         = label_elem.parent('.c-expander'),
					elem_panel          = label_elem.next('.c-expander__panel'),
					is_mobile_ex        = (elem_parent.hasClass('c-expander--navigation')) ? true : false,
					is_mobile_target    = Modernizr.mq('(max-width: 767px)');

			// if the panel is hidden open it - check if it's a nagivation @mobile only expander
			// else, close it
			if (!is_mobile_ex && elem_parent.attr('data-state') == 'is-hidden' || is_mobile_ex && is_mobile_target && elem_parent.attr('data-state') == 'is-hidden') {

					// open the one we want
					elem_parent.attr('data-state', 'is-expanded');

					// if this is part of a group, close all the other members
					if(elem_parent.attr('data-group') !== undefined) {
							var datagroup = elem_parent.attr('data-group');
							$('[data-group='+ datagroup +']').not(elem_parent).each(function() {
									$(this).attr('data-state', 'is-hidden');
							});
					}

					// if the content is pushed out of view when expanded, scroll it into view
					elem_panel.one(transitionEvent, function (event) {
						if (!isScrolledIntoView(elem_parent)) {
							console.log(elem_parent.attr("id"));
							backToTop(elem_parent.attr("id"));
						}
					});

			} else {
					// change to animation state
					elem_parent.attr('data-state', 'is-animating');

					// wait for css transition to end before changing state back to hidden
					// otherwise things can get a little jumpy
					elem_panel.one(transitionEvent, function (event) {
							elem_parent.attr('data-state', 'is-hidden');
					});
			}

			// Reset the hidden state on the navigation @mobile only expander if
			// it's @tablet and above screen sizes, otherswise the label will still be clickable
			// and mess things up.
			if (is_mobile_ex && !is_mobile_target && elem_parent.attr('data-state') != 'is-hidden') {
					elem_parent.attr('data-state', 'is-hidden');
			}

	});
}



/* Page loaded functions
	 ========================================================================== */

$(window).on('load', function () {

	// Attach function call to links with this class name.
	// $(".toTop").on('click', function (e) {
	// 	e.preventDefault();
	// 	backToTop();
	// });

	// Initialize the expander component if it's present
	if($('.c-expander').exists()) {
		res_expander();
	}

	// Check if there is a hash tag on the url and scroll to the container with
	// the matching ID.
	var hasHashScroll = (window.location.hash) ? setTimeout(function() { scrollToAsset(window.location.hash.substring(1)) }, 500) : false;

});
