/* ==========================================================================
   #SCRIPTS
   ========================================================================== */
/**
 * Some default js functions that we might commonly use.
 * Keep what you need, toss the rest.
 *
 * CONTENTS
 *
 * exists..................Use this function in your scripts if you need to check
 *                         whether an element exists on the page before running.
 *                         This can help avoid js errors.
 *
 * hasAttr.................Check if an element has an attribute.
 *
 * isScrolledIntoView......Check when an element is scrolled into the viewport
 *                         area. Used by res_expander().
 *
 * whichAnimationEvent.....Listener to help with CSS transitions and animations.
 *                         Used by res_expander().
 */


// See if an element exists in the DOM
$.fn.exists = function(){ return this.length>0; };

// Check if an element has a particular attribute
$.fn.hasAttr = function (attr) { var attribVal = this.attr(attr); return (attribVal !== undefined) && (attribVal !== false); };

// Helper function to determine when/if an element enters or leaves the viewport
// to then trigger an action/event
function isScrolledIntoView(elem) {
    var $elem = $(elem);
    var $window = $(window);

    var docViewTop = $window.scrollTop();
    var docViewBottom = docViewTop + $window.height();

    var elemTop = $elem.offset().top;
    var elemBottom = elemTop + $elem.height();

    return ((elemBottom <= docViewBottom) && (elemTop >= docViewTop));
}



// Function from David Walsh: http://davidwalsh.name/css-animation-callback
// make sure browser is listening for only one version of event
function whichAnimationEvent() {
    var t,
	el = document.createElement("fakeelement");
    var animations = {
        "animation": "animationend",
        "OAnimation": "oAnimationEnd",
        "MozAnimation": "animationend",
        "WebkitAnimation": "webkitAnimationEnd"
    }
    for (t in animations) {
        if (el.style[t] !== undefined) {
            return animations[t];
        }
    }
}
var animationEvent = whichAnimationEvent();

function whichTransitionEvent() {
    var t,
        el = document.createElement("fakeelement");
    var transitions = {
        "transition": "transitionend",
        "OTransition": "oTransitionEnd",
        "MozTransition": "transitionend",
        "WebkitTransition": "webkitTransitionEnd"
    }

    for (t in transitions) {
        if (el.style[t] !== undefined) {
            return transitions[t];
        }
    }
}
var transitionEvent = whichTransitionEvent();

/* Page loaded functions
   ========================================================================== */
$(window).on('load', function () {
    $('.c-survey').delegate('.js-choose-answer', 'click', function (e) {
        e.preventDefault();

        var link_elem   = $(this),
        link_state      = link_elem.attr('data-state'),
        is_other        = link_elem.attr('data-other'),
        elem_parent     = link_elem.parents('.c-survey'),
        is_multi_select = elem_parent.attr('data-multi-select');

        // If not a multi-select screen, deselect a previously selected answer
        if (is_multi_select == 'false') {
            $('.c-survey__answer').attr('data-state', '');

            // Hide description field if deselecting `Other`
            if(is_other != 'true') {
                $('.c-survey__item--other').attr('data-state', 'is-hidden');
            }

        }

        // Check if answer is already selected
        if(link_state == 'is-active') {
            
            // De-select answer if it's already been selected
            link_elem.attr('data-state', '');
            
            // Hide description field if deselecting `Other`
            if(is_other == 'true') {
                $('.c-survey__item--other').attr('data-state', 'is-hidden');
            }

        } else {
            
            // Highlight selected answer
            link_elem.attr('data-state', 'is-active');

            // Show description field if selecting `Other`
            if(is_other == 'true') {
                $('.c-survey__item--other').attr('data-state', "is-active");
            }
        }
    });

    $('.c-survey__icon-list').delegate('.js-choose-icon', 'click', function (e) {
        e.preventDefault();
        iconTapped($(this));
    });    

    $('#bmi-form').delegate('.js-bmi-select', 'change', function (e) {
        e.preventDefault();
        bmiElementsChanged(e);
    });

    $('.c-survey__change-units').delegate('a', 'click', function (e) {
        e.preventDefault();
        changeUnits();
    });

    $('.c-btn--save-answer').click(function (e) {
        e.preventDefault();
        sendAnswersToJavaScriptInterface(e);
    });
});

/* Content injection functions
   ========================================================================== */

function insertStepInformation(stepIndex, stepCount) {
    clearStepInformation();

    $('#btn-survey1_next').show();

    if (stepIndex === stepCount - 1) {
        $('#btn-survey1_save').hide();
    } else {
        $('#btn-survey1_save').show();
    }

    if (stepCount === 1) {
        return;
    }

    for (var i = 0; i < stepCount; ++i) {
        var div = document.createElement('div');
        div.classList.add('c-survey__step-indicator');
        div.setAttribute('data-step', stepIndex + 1);
        div.setAttribute('data-state', i <= stepIndex ? 'is-active' : '');
        $('.c-survey__stepper').append(div);
    }
}

function clearStepInformation() {
    $('#btn-survey1_next').hide();
    $('#btn-survey1_save').hide();
    $('.c-survey__stepper').empty();
};

function insertQuestion(question) {
    $('.c-survey__question').text(question);
}

function insertHint(hint) {
    $('.c-survey__hint').text(hint);
}

function sendAnswersToJavaScriptInterface(event) {
    var answers = getAnswers();
    var finishLater;
    if (event !== undefined)
    {
        switch (event.target.id) {
            case 'btn-survey1_next':
                finishLater = false;
                break;
            case 'btn-survey1_save':
                finishLater = true;
                break;
            default:
                finishLater = false;
                break;
        }
    }
    else
    {
        finishLater = false;
    }    

    saveAnswers(answers, finishLater);
}

function hideButtons() {
    $('#btn-survey1_save').hide();
    $('#btn-survey1_next').hide();
}
